#!/bin/bash

# 不需要压缩的文件
FILE_WHITE_LISTS=("base64.js" "md5.js" "rsa.js")

# 不压缩的文件夹
DIR_WHITE_LISTS=("node_modules" "")

#js压缩
GOOGLE_CLOSURE_COMPILER_JS="google-closure-compiler-js"

#打包
ASAR="asar"

#安装所需组件
#     * @param 
#     * @returns void
function install_relies() {
    echo "Start to install relies..."

    is_exist_compiler=`which $GOOGLE_CLOSURE_COMPILER_JS`
    if [ "$is_exist_compiler" == "" ]; then
        echo "Need to install "$GOOGLE_CLOSURE_COMPILER_JS
        sudo npm install -g $GOOGLE_CLOSURE_COMPILER_JS
        if [ $? -eq 0 ];then
            echo $GOOGLE_CLOSURE_COMPILER_JS" was installed successfully!"
        else
            echo  -e "\033[31m Fail to install "$GOOGLE_CLOSURE_COMPILER_JS"\033[0m"
        fi
    else
        echo $GOOGLE_CLOSURE_COMPILER_JS" exists."
    fi

    is_exist_asar=`which $ASAR`
    if [ "$is_exist_asar" == "" ]; then
        echo "Need to install "$ASAR
        sudo npm install -g $ASAR
        if [ $? -eq 0 ];then
            echo $ASAR" was installed successfully!"
        else
            echo  -e "\033[31m Fail to install "$ASAR"\033[0m"
        fi
    else
        echo $ASAR" exists."
    fi

    echo "Finished installing relies."

}

#压缩文件
#     * @param $1: 文件夹
#     *        $2: 文件名
#     * @returns void
function compress() {
    echo "Compressing "$1"/"$2
    google-closure-compiler-js --compilationLevel=WHITESPACE_ONLY --languageIn=ES6 --languageOut=ES6 $1"/"$2 > $1"/min_"$2
    if [ $? -eq 0 ];then
        cp $1"/min_"$2 $1"/"$2
        rm $1"/min_"$2
    else
        rm "/min_"$2
        echo  -e "\033[31m""Fail to compress "$1"/"$2"\033[0m"
    fi
}

#遍历文件夹
#     * @param $1: 文件夹
#     *      
#     * @returns void
function get_dir() {
    for element in `ls $1`
    do  
        dir_or_file=$1"/"$element
        if [ -d $dir_or_file ]
        then 
            [[ "${DIR_WHITE_LISTS[*]}" =~ ${element} ]] || get_dir $dir_or_file
        else    
            if [[ "${element##*.}"x = "js"x ]]
            then
                [[ "${FILE_WHITE_LISTS[*]}" =~ ${element} ]] || compress $1 $element
            fi
        fi
    done
    
}

#asar打包
#     * @param $1: 打包的文件夹
#     *        $2: 输出文件夹
#     * @returns void
function asar_dir() {
    asar p $1 $2"/"app.asar
    if [ $? -eq 0 ]; then
        echo "asar app successfully."
    else
        echo -e "\033[31m asar app failed.\033[0m"
    fi
}

#记住当前路径，用户执行完脚本后恢复
curr_dir=`pwd`

install_relies

#获取脚本路径
cd `dirname $0`
cd ../..
#拷贝app目录
cp -rf ./app app_tmp
cd app_tmp
start_dir=`pwd`

#删除svn信息
rm -rf .svn
if [ $? -eq 0 ]; then
    echo "rm .svn successfully"
else
    echo -e "\033[31m rm .svn failed.\033[0m"
fi

echo "Start compressing..."
get_dir $start_dir
echo "Finish compressing."

echo "Start asar..."
asar_dir $start_dir $start_dir"/.."
echo "Finish asar."

cd ..
rm -rf app_tmp

#回到脚本执行前的路径
cd $curr_dir

echo "All Finished"
