'use strict';const Url=require("url");const Https=require("https");const IPCMain=require("electron").ipcMain;const Shell=require("electron").shell;const App=require("electron").app;const Common=require("../common");const PreAuth=require("../service/auth/pre_auth");const CertAuth=require("../service/auth/cert_auth");const PasswordAuth=require("../service/auth/password_auth");const SubmitNewPsw=require("../service/auth/submit_new_psw");const RandCodeAuth=require("../service/auth/rand_code_auth");
const HardIDAuth=require("../service/auth/hard_id_auth");const SmsAuth=require("../service/auth/sms_auth");const ChallengeAuth=require("../service/auth/challenge_auth");const TokenAuth=require("../service/auth/token_auth");const Logout=require("../service/auth/logout");const ECEvent=require("../service/lib/ec_event");const Log=require("../service/lib/ec_log");const ECUtil=require("../service/lib/ec_util");const SessionQuerryTimer=require("../service/lib/session_querry_timer");const RSAKey=require("../service/lib/rsa");
const ECI18n=require("../service/lib/ec_i18n");function AuthController(){this.mIsInited=false;this.mGetConfOk=false;this.mGetRCListOk=false;this.mOpenResources=false}AuthController.prototype.init=function(){if(this.mIsInited)return;Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Starting initializing AuthController...");this.initEvent();if(!gStartFromECAgent)this.initIpcMain();this.mIsInited=true;Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Initialize AuthController successfully!")};
AuthController.prototype.createAuth=function(type){switch(type){case Common.CERT_AUTH:return new CertAuth;case Common.PASSWORD_AUTH:return new PasswordAuth;case Common.SMS_AUTH:return new SmsAuth;case Common.HARDWARE_ID_AUTH:return new HardIDAuth;case Common.USBKEY_AUTH:return null;case Common.CHALLENGE_AUTH:return new ChallengeAuth;case Common.TOKEN_AUTH:return new TokenAuth;default:return null}};
AuthController.prototype.initEvent=function(){Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Starting initializing ec_event...");ECEvent.on(Common.EVENT_PRE_AUTH_RESULT,(resStr)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_PRE_AUTH_RESULT);if(resStr==null||""==resStr){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The pre_auth_result is null!");return}Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,Common.EVENT_PRE_AUTH_RESULT+": "+resStr);this.doPreAuthRes(resStr)});ECEvent.on(Common.EVENT_IMPORT_CERT_RES,
(resObj)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_IMPORT_CERT_RES);if(resObj==null||!("result"in resObj))Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The result is null!");var errMsg=null;switch(resObj["result"]){case Common.IMPORT_CERT_SUCCESS:Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"The cert was imported successfully!");break;case Common.IMPORT_CERT_ERROR_FILE_PATH:errMsg=ECI18n("Certificate path is incorrect!");break;case Common.IMPORT_CERT_ERROR_PASSWORD:errMsg=
ECI18n("Certificate password is incorrect!");break;case Common.IMPORT_CERT_ERROR_UNSUPPORT_FORMAT:case Common.IMPORT_CERT_ERROR_UNKNOWN_FORMAT:errMsg=ECI18n("Certificate formate is incorrect!");break;case Common.IMPORT_CERT_ERROR_IS_EXIST:errMsg=ECI18n("Certificate exists!");break;default:errMsg=ECI18n("Import failed!");break}if(errMsg!=null){Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Import Cert error: "+resObj["result"]);ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:errMsg})}});
ECEvent.on(Common.EVENT_GET_CERTIFICATES_OK,()=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_GET_CERTIFICATES_OK);Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"get certificates ok!");var currCert=null;Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"certs: "+JSON.stringify(gAuthInfo["certs"]));Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"oldCerts: "+JSON.stringify(gAuthInfo["oldCerts"]));var newCertNum=0;for(var certsID in gAuthInfo["certs"])if(!(certsID in gAuthInfo["oldCerts"])){if(currCert==
null)currCert=certsID;newCertNum++}if(newCertNum>1){var confLastCert=gControllers.conf.get("last_cert");if(confLastCert!=null&&confLastCert!="")currCert=confLastCert}if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_UPDATE_CERTS,JSON.stringify({"currCert":currCert}))});ECEvent.on(Common.EVENT_AUTH_ERROR,(errCode)=>{Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_AUTH_ERROR+" errCode: "+errCode);Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Login failed. Error code: "+
errCode);if(gStartFromECAgent){ECUtil.doLogout();return}gAuthInfo[Common.AUTHINFO_IS_FIRST_PSW_AUTH]=false;gAuthInfo[Common.AUTHINFO_IS_FIRST_CERT_AUTH]=false;var errMsg="",desWindow=Common.LINE_WINDOW;if(gCurrState<Common.STATE_AUTH){errMsg=ECI18n("Failed to connect to server. Please check whether the network works well.");desWindow=Common.LINE_WINDOW}else if(errCode==Common.AUTH_REQUEST_ERROR){errMsg=ECI18n("Failed to connect to server. Please check whether the network works well.");desWindow=Common.AUTH_WINDOW}else{errMsg=
ECI18n("Login failed!");desWindow=Common.AUTH_WINDOW}ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:errMsg});gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,desWindow)});ECEvent.on(Common.EVENT_CERT_AUTH_ERROR,(errCode)=>{Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_CERT_AUTH_ERROR+" errCode: "+errCode);Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Cert Auth Error. Error code: "+errCode);gAuthInfo[Common.AUTHINFO_IS_FIRST_PSW_AUTH]=
false;gAuthInfo[Common.AUTHINFO_IS_FIRST_CERT_AUTH]=false;ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:ECI18n("Login failed!")});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CERT_AUTH_ERROR,null,errCode)});ECEvent.on(Common.EVENT_AUTH_RESULT,(resStr)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_AUTH_RESULT);if(resStr==null||""===resStr){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The auth_result is null!");return}Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,
Common.EVENT_AUTH_RESULT+": "+resStr);this.doAuthRes(resStr)});ECEvent.on(Common.EVENT_NEW_PSW_GET_PAGE_OK,()=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_NEW_PSW_GET_PAGE_OK);if(gECWindow!=null){gECWindow.openMdfPswWindow();gECWindow.hide()}});ECEvent.on(Common.EVENT_SUBMIT_NEW_PSW_RESULT,(resStr)=>{Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_SUBMIT_NEW_PSW_RESULT+" resStr: "+resStr);if(gECWindow!=null)gECWindow.closeMdfPswWindow();
if(resStr===Common.SUBMIT_NEW_PSW_OK){Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Submit new password successfully!");ECUtil.showMessageBox(Common.DIALOG_SHOW_AS_INDIVIDUAL_WINDOW,{type:"info",title:ECI18n("Prompt"),message:ECI18n("Modify password successfully!")});this.doAuthOK({curAuth:Common.PASSWORD_AUTH})}else{ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:ECI18n("Failed to modify the password!")});if(gECWindow!=null){gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,
null,Common.AUTH_WINDOW);gECWindow.show()}}});ECEvent.on(Common.EVENT_SUBMIT_PHONE_RESULT,(resStr)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_SUBMIT_PHONE_RESULT);if(resStr==null||""===resStr){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The submit_phone_result is null!");return}this.doSubmitPhoneRes(resStr)});ECEvent.on(Common.EVENT_REGET_SMS_CODE_RESULT,(resStr)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_REGET_SMS_CODE_RESULT);if(resStr==
null||""===resStr){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The reget_sms_code_result is null!");return}this.doRegetSmsCodeRes(resStr)});ECEvent.on(Common.EVENT_GET_SMS_PAGE_RESULT,(disableTime,needLoadUrl)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_GET_SMS_PAGE_RESULT);if(needLoadUrl===Common.SMS_AUTH_NEED_LOAD_URL){gAuthInfo["disableTime"]=disableTime;if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,Common.AUTH_WINDOWS[gAuthInfo["curAuth"]],
Common.SMS_WINDOW);return}else if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_SMS_DISABLE_TIME,disableTime)});ECEvent.on(Common.EVENT_GET_HARDID_OK,()=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_GET_HARDID_OK);setTimeout(()=>{(new HardIDAuth(Common.HARDWARE_ID_AUTH_GETTYPE)).doRequestAuth(gAuthInfo["authUrl"],null)},10)});ECEvent.on(Common.EVENT_HARD_ID_COLLECTION_RESULT,(resStr)=>{Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+
Common.EVENT_HARD_ID_COLLECTION_RESULT);if(resStr==null||""===resStr){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The hard_id_collection_result is null!");return}Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"event_hard_id_collection_result: "+resStr);var resObj=JSON.parse(resStr);if(resObj["startWithGetType"])if(resObj["result"]===Common.HARD_ID_COLLECTION_OK||resObj["result"]===Common.HARD_ID_COLLECTION_HAVE_SUB||resObj["result"]===Common.HARD_ID_HAVE_VERIFIED||resObj["result"]===Common.HARD_ID_AUTH_AUTO_APPROVAL){if(gECWindow==
null){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"gECWindow is null!");return}var errPat=/Location(\s*)=(\s*)(.*pwp_errorcode(\s*)=(\s*)(\d+).*)/;var errMatches=errPat.exec(resObj["note"]);if(errMatches!=null)this.doAuthError({curAuth:Common.HARDWARE_ID_AUTH,note:resObj["note"]});else this.doAuthOK({curAuth:Common.HARDWARE_ID_AUTH})}else if(resObj["result"]===Common.HARD_ID_COLLECTION_TIMEOUT){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Prompt"),message:ECI18n("Session times out. Please log in again.")});
if(gECWindow!=null)gECWindow.getWebContents.send(Common.EVENT_MAIN_CHANGE_WINDOW,Common.AUTH_WINDOWS[gAuthInfo["curAuth"]],Common.AUTH_WINDOW)}else if(resObj["result"]===Common.HARD_ID_OVER_MAX){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Prompt"),message:ECI18n("Number of hardware ID has reached the upper limit. Please contact the administrator.")});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}else if(resObj["result"]===
Common.HARD_ID_OVER_MAX_USERS){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Prompt"),message:ECI18n("Number of licensed users has reached the upper limit!")});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}else if(resObj["result"]===Common.HARD_ID_OVER_MAX_GROUP_USERS){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Prompt"),message:ECI18n("Number of concurrent users of the parent user group has reached the upper limit!")});
if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}else{Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"Cannot understand the hard_id_collection result: "+resObj["result"]);ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:ECI18n("Login failed!")});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}else if(resObj["result"]===Common.HARD_ID_COLLECTION_OK||resObj["result"]===
Common.HARD_ID_COLLECTION_HAVE_SUB){if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,Common.HARDID_SUBMIT_WINDOW,Common.HARDID_SUCCESS_WINDOW)}else if(resObj["result"]===Common.HARD_ID_AUTH_AUTO_APPROVAL){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"The hard id is approved automatically!");var errPat=/Location(\s*)=(\s*)(.*pwp_errorcode(\s*)=(\s*)(\d+).*)/;var errMatches=errPat.exec(resObj["note"]);if(errMatches!=null)this.doAuthError({curAuth:Common.HARDWARE_ID_AUTH,
note:resObj["note"]});else this.doAuthOK({curAuth:Common.HARDWARE_ID_AUTH})}else if(resObj["result"]===Common.HARD_ID_COLLECTION_TIMEOUT){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Prompt"),message:ECI18n("Session times out. Please log in again.")});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,Common.HARDID_SUBMIT_WINDOW,Common.AUTH_WINDOW)}else if(resObj["result"]===Common.HARD_ID_OVER_MAX){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Prompt"),
message:ECI18n("Number of hardware ID has reached the upper limit. Please contact the administrator.")});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}else if(resObj["result"]===Common.HARD_ID_OVER_MAX_USERS){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Prompt"),message:ECI18n("Number of licensed users has reached the upper limit!")});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}else if(resObj["result"]===
Common.HARD_ID_OVER_MAX_GROUP_USERS){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Prompt"),message:ECI18n("Number of concurrent users of the parent user group has reached the upper limit!")});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}else{Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"Cannot understand the hard_auth result: "+resObj["result"]);ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:ECI18n("Login failed!")});
if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}});ECEvent.on(Common.EVENT_SET_SERVADDR_OK,()=>{Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_SET_SERVADDR_OK);this.setTWFIDToECAgent()});ECEvent.on(Common.EVENT_CONFIGURE_TWFID_OK,()=>{Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_CONFIGURE_TWFID_OK);this.mGetConfOk=false;this.mGetRCListOk=false;if(this.mGetConfOk&&this.mGetRCListOk){this.startService();
return}if(!this.mGetConfOk)gControllers.httpsCom.requestECAgent("GetConfig",{arg1:"1",token:gAuthInfo["currToken"],callback:Common.GET_CONF_CALLBACK});if(!this.mGetRCListOk)gControllers.httpsCom.requestECAgent("GetConfig",{arg1:"2",token:gAuthInfo["currToken"],callback:Common.GET_RC_LIST_CALLBACK})});ECEvent.on(Common.EVENT_GET_CONF_OK,()=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_GET_CONF_OK);if(gStartFromECAgent){var selectLineUrl=null;if(gConf!=null&&"Conf"in gConf&&
"Webagent"in gConf.Conf)if(gConf.Conf.Webagent["enable"]==="1")if(gConf.Conf.Webagent["address"]!=="")selectLineUrl=gConf.Conf.Webagent["address"];if(selectLineUrl==null)gControllers.httpsCom.requestECAgent(Common.HTTPS_DO_QUERY,{arg1:"QUERY LOGINADDR",callback:Common.QUERRY_LOGIN_ADDR_CALLBACK});else{gAuthInfo[Common.AUTHINFO_SELECT_LINE_URL]=selectLineUrl;gControllers.conf.addUrl(selectLineUrl);gControllers.conf.save()}}if(!this.mGetConfOk){this.mGetConfOk=true;if(this.mGetConfOk&&this.mGetRCListOk){if(!gStartFromECAgent)this.showPswRemind();
this.startService()}}});ECEvent.on(Common.EVENT_GET_RCLIST_OK,()=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_GET_RCLIST_OK);if(!this.mGetRCListOk){this.mGetRCListOk=true;if(this.mGetConfOk&&this.mGetRCListOk){if(!gStartFromECAgent)this.showPswRemind();this.startService()}}});ECEvent.on(Common.EVENT_START_SERVICE_OK,()=>{Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_START_SERVICE_OK);this.doStartServiceOk()});ECEvent.on(Common.EVENT_START_SERVICE_ERROR,
()=>{Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_START_SERVICE_ERROR);if(gStartFromECAgent)ECUtil.exit(1);else{if(gECTray!=null){gECTray.destroy();gECTray=null}gControllers.httpsCom.requestECAgent("Logout",{callback:"this.logoutCallback"});ECEvent.once(Common.EVENT_START_SERVICE_ERROR_LOGOUT_OK,function(){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:ECI18n("Service failed to start. Please try logining in again.")})});(new Logout(Common.LOGOUT_START_SERVICE_ERROR)).doRequestAuth(gAuthInfo["authUrl"],
null);if(gECWindow!=null){gECWindow.show();gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}}});ECEvent.on(Common.EVENT_QSTATE_RESULT,(service,status)=>{Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Received ec_event: "+Common.EVENT_QSTATE_RESULT+" service: "+service+" status : "+status);if(service==null||service===""){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The value of service is null");return}var isServiceOk=false;switch(status){case Common.STATUS_LOGOUT:case Common.TOKEN_CHECK_ERROR:isServiceOk=
"logout";break;case Common.STATUS_STARTING:case Common.STATUS_STOPPING:case Common.STATUS_INITING:case Common.STATUS_INIT_SUCCESS:isServiceOk=false;break;case Common.STATUS_START_SUCCESS:case Common.STATUS_STARTED:isServiceOk=true;break;case Common.STATUS_START_FAILED:case Common.STATUS_STOP_SUCCESS:case Common.STATUS_STOPPED:case Common.STATUS_INIT_FAILED:isServiceOk=null;break}if(isServiceOk==="logout"){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"doStartServiceLogout");this.doStartServiceLogout()}else if(isServiceOk==
null)this.doStartServiceError();else if(service===Common.QSTATE_ALLSERVICES)gIsAllServiceOk=isServiceOk;else if(service===Common.QSTATE_L3VPN)gIsL3VPNOk=isServiceOk});Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Initialize ec_event successfully!")};
AuthController.prototype.initIpcMain=function(){IPCMain.on(Common.EVENT_RENDER_IMPROT_CERT,(event,args)=>{gControllers.httpsCom.requestECAgent(Common.HTTPS_GET_ENCRYPT_KEY,{callback:Common.GET_ENCRYPT_KEY_FOR_CERT_AND_TWFID});ECEvent.removeAllListeners(Common.EVENT_GET_ENCRYPT_KEY_FOR_CERT_AND_TWFID_RES);ECEvent.once(Common.EVENT_GET_ENCRYPT_KEY_FOR_CERT_AND_TWFID_RES,(res)=>{if(res!==Common.GET_ENCRYPT_KEY_OK){ECEvent.emit(Common.EVENT_IMPORT_CERT_RES,{result:Common.IMPORT_CERT_ERROR_GET_ENCRYPT_KEY_ERROR});
return}var argsObj=JSON.parse(args);var encryptKey=gECAgent["encryptKey"],encryptExp=parseInt("65537").toString(16);var rsaKey=new RSAKey;rsaKey.setPublic(encryptKey,encryptExp);argsObj["password"]=rsaKey.encrypt(Common.HTTPS_ECAGENT_CERT_PSW_SALT+argsObj["password"]);Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,argsObj["password"]);gControllers["httpsCom"].requestECAgent("ImportCert",{arg1:argsObj["filename"],arg2:argsObj["password"],callback:Common.IMPORT_CERT_CALLBACK})})});IPCMain.on(Common.EVENT_RENDER_IMPORT_CERT_PSW,
(event)=>{if(gECWindow!=null)gECWindow.openCertPswWindow()});IPCMain.on(Common.EVENT_RENDER_IMPORT_CERT_CONFIRM_OR_CLOSE,(event)=>{if(gECWindow!=null)gECWindow.closeCertPswWindow()});IPCMain.on(Common.EVENT_RENDER_REMOVE_CERT,(event,certLabel,serialNum)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+Common.EVENT_RENDER_REMOVE_CERT+" certLabel : "+certLabel+" serialNum: "+serialNum);var delCert=certLabel+serialNum;if(gAuthInfo[Common.AUTHINFO_CERTS]!=null&&delCert in gAuthInfo[Common.AUTHINFO_CERTS]){delete gAuthInfo[Common.AUTHINFO_CERTS][delCert];
gAuthInfo[Common.AUTHINFO_CERTS]=JSON.parse(JSON.stringify(gAuthInfo[Common.AUTHINFO_CERTS]))}if(gControllers.httpsCom!=null)gControllers.httpsCom.requestECAgent(Common.HTTPS_REMOVE_CERT,{arg1:certLabel,arg2:serialNum})});IPCMain.on(Common.EVENT_RENDER_CHANGE_SERVER,(event)=>{gCurrState=Common.LINE_WINDOW;this.clearGlobals();if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,Common.AUTH_WINDOW,Common.LINE_WINDOW)});IPCMain.on(Common.EVENT_RENDER_CERT_AUTH,(event,args)=>
{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+Common.EVENT_RENDER_CERT_AUTH+" args : "+args);(new CertAuth).doRequestAuth(gAuthInfo["authUrl"],args)});IPCMain.on(Common.EVENT_RENDER_PASSWORD_AUTH,(event,args)=>{var argsObj=JSON.parse(args);gAuthInfo.userName=argsObj[Common.USER_NAME];gAuthInfo.psw=argsObj[Common.PASSWORD];(new PasswordAuth).doRequestAuth(gAuthInfo["authUrl"],argsObj)});IPCMain.on(Common.EVENT_RENDER_SUMBIT_NEW_PSW,(event,args)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,
"Redeived ipcMain_event: "+Common.EVENT_RENDER_SUMBIT_NEW_PSW+" args : "+args);var argsObj=JSON.parse(args);(new SubmitNewPsw).doRequestAuth(gAuthInfo["authUrl"],argsObj)});IPCMain.on(Common.EVENT_RENDER_SMS_AUTH,(event,smsCode)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+Common.EVENT_RENDER_SMS_AUTH+" smsCode : "+smsCode);(new SmsAuth).doRequestAuth(gAuthInfo["authUrl"],{svpn_inputsms:smsCode})});IPCMain.on(Common.EVENT_RENDER_SMS_CODE_REGET,(event,argsObj)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,
"Redeived ipcMain_event: "+Common.EVENT_RENDER_SMS_CODE_REGET);(new SmsAuth).doRequestAuth(gAuthInfo["authUrl"],argsObj)});IPCMain.on(Common.EVENT_RENDER_SUBMIT_PHONE,(event,phone)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+Common.EVENT_RENDER_SUBMIT_PHONE+" phone: "+phone);(new SmsAuth).doRequestAuth(gAuthInfo["authUrl"],{phone_number:phone})});IPCMain.on(Common.EVENT_RENDER_SMS_AUTH_BOUND_PHONE,(event,smsCode)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+
Common.EVENT_RENDER_SMS_AUTH_BOUND_PHONE+" smsCode: "+smsCode);(new SmsAuth).doRequestAuth(gAuthInfo["authUrl"],{bound_phone_code:smsCode})});IPCMain.on(Common.EVENT_RENDER_HARD_ID_AUTH,(event)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+Common.EVENT_RENDER_HARD_ID_AUTH);(new HardIDAuth(Common.HARD_ID_AUTH_COLLECTION)).doRequestAuth(gAuthInfo["authUrl"],gAuthInfo["hardID"])});IPCMain.on(Common.EVENT_RENDER_CHALLENGE_AUTH,(event,challengeCode)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,
"Redeived ipcMain_event: "+Common.EVENT_RENDER_CHALLENGE_AUTH);(new ChallengeAuth).doRequestAuth(gAuthInfo["authUrl"],{svpn_inputradius:challengeCode})});IPCMain.on(Common.EVENT_RENDER_TOKEN_AUTH,(event,tokenCode)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+Common.EVENT_RENDER_TOKEN_AUTH);(new TokenAuth).doRequestAuth(gAuthInfo["authUrl"],{svpn_inputtoken:tokenCode})});IPCMain.on(Common.EVENT_RENDER_RAND_CODE_AUTH,(event)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+
Common.EVENT_RENDER_RAND_CODE_AUTH);(new RandCodeAuth).doRequestAuth(gAuthInfo["authUrl"])});IPCMain.on(Common.EVENT_RENDER_MODIFY_PASSWORD_CLOSE,(event)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+Common.EVENT_RENDER_MODIFY_PASSWORD_CLOSE);if(gECWindow!=null)gECWindow.closeMdfPswWindow();if(gControllers.httpsCom!=null)gControllers.httpsCom.requestECAgent("Logout",{callback:"this.logoutCallback"});(new Logout(null)).doRequestAuth(gAuthInfo["authUrl"],null)});IPCMain.on(Common.EVENT_RENDER_MODIFY_PASSWORD_MINIMIZE,
(event)=>{Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Redeived ipcMain_event: "+Common.EVENT_RENDER_MODIFY_PASSWORD_MINIMIZE);if(gECWindow!=null)gECWindow.minimizeMdfPswWindow()})};AuthController.prototype.startAuth=function(){(new PreAuth).doRequestAuth(gAuthInfo["authUrl"])};
AuthController.prototype.clearGlobals=function(){Cookies.remove(gAuthInfo["authUrl"],"ENABLE_RANDCODE",(error)=>{if(error)Log.warning(Common.LOG_EC_UTIL_TAG,"remove cookie(ENABLE_RANDCODE) of "+gAuthInfo["authUrl"]+" error: "+error)});gAuthInfo[Common.AUTHINFO_SELECT_LINE_URL]=null;gAuthInfo[Common.AUTHINFO_AUTH_URL]=null;gAuthInfo[Common.AUTHINFO_PRE_TOKEN]=null;gAuthInfo[Common.AUTHINFO_CURR_TOKEN]=null;gAuthInfo[Common.AUTHINFO_SMS_SEND_TYPE]=null;gAuthInfo[Common.AUTHINFO_HARD_ID]=null;gAuthInfo[Common.AUTHINFO_OLD_CERTS]=
{};gAuthInfo[Common.AUTHINFO_CERTS]={};gAuthInfo[Common.AUTHINFO_USER_NAME]=null;gAuthInfo[Common.AUTHINFO_PSW]=null;gAuthInfo[Common.AUTHINFO_IS_FIRST_AUTH]=true;gAuthInfo[Common.AUTHINFO_IS_FIRST_PSW_AUTH]=true;gAuthInfo[Common.AUTHINFO_IS_FIRST_CERT_AUTH]=true;gAuthInfo[Common.AUTHINFO_MODIFY_PSW]={};gAuthInfo.modifyPsw["errCode"]=null;gAuthInfo.modifyPsw["minPswLen"]=0;gAuthInfo.modifyPsw["isPswExpire"]=0;gAuthInfo.modifyPsw["isFstLogin"]=0;gAuthInfo.modifyPsw["isLimitLen"]=0;gAuthInfo.modifyPsw["isNotName"]=
0;gAuthInfo.modifyPsw["isMustNum"]=0;gAuthInfo.modifyPsw["isMustChar"]=0;gAuthInfo.modifyPsw["isMustSchar"]=0;gAuthInfo.modifyPsw["isDifOld"]=0;gAuthInfo[Common.AUTHINFO_NEED_START_L3VPN]=false;gAuthInfo[Common.AUTHINFO_DISABLE_TIME]=0;gAuthInfo[Common.AUTHINFO_PHONE]=null;gAuthInfo[Common.AUTHINFO_CURR_AUTH]=null;gAuthInfo[Common.AUTHINFO_NEXT_AUTH]=null;gAuthInfo[Common.AUTHINFO_CHALLENGE_MSG]=null;gAuthInfo[Common.AUTHINFO_IMPORT_CERT]={};gAuthInfo[Common.AUTHINFO_IMPORT_CERT]["file"]=null;gAuthInfo[Common.AUTHINFO_IMPORT_CERT]["pswWindow"]=
null};
AuthController.prototype.doPreAuthRes=function(resStr){if(resStr==null||""==resStr)return;var resObj=JSON.parse(resStr);var lastStartAuth=gControllers["conf"].get("last_start_auth");gRender["startAuth"]=Common.PASSWORD_AUTH;Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"lastStartAuth: "+lastStartAuth);if(lastStartAuth!=null)gRender["startAuth"]=lastStartAuth;else if(resObj["startAuth"]!=null)gRender["startAuth"]=resObj["startAuth"];Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"gRender[startAuth]: "+gRender["startAuth"]);
Cookies.get({url:gAuthInfo["authUrl"],name:"ENABLE_RANDCODE"},function(error,cookies){gPreAuth[Common.PRE_AUTH_RNDIMG]=false;if(error){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"gCookies: error"+error);gPreAuth[Common.PRE_AUTH_RNDIMG]=false}else if(cookies.length>0)if(cookies[0]!=null&&cookies[0]["value"]==="1")gPreAuth[Common.PRE_AUTH_RNDIMG]=true;if(gPreAuth[Common.PRE_AUTH_RNDIMG])(new RandCodeAuth).doRequestAuth(gAuthInfo["authUrl"]);gCurrState=Common.STATE_AUTH;if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,
Common.LINE_WINDOW,Common.AUTH_WINDOW)})};
AuthController.prototype.doAuthRes=function(resStr){if(resStr==null||""==resStr)return;var resObj=JSON.parse(resStr);switch(resObj["result"]){case Common.AUTH_RESULT_FAILED:Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"Auth failed!");this.doAuthError(resObj);break;case Common.AUTH_RESULT_OK:Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Auth successfully!");this.doAuthOK(resObj);break;case Common.AUTH_RESULT_NEXT:Log.info(Common.LOG_AUTH_CONTROLLER_TAG,"Auth successfully and have another auth!");this.doNextAuth(resObj);
break;default:break}};
AuthController.prototype.doAuthError=function(resObj){if(typeof resObj!=="object"||resObj==null)return;gAuthInfo[Common.AUTHINFO_IS_FIRST_PSW_AUTH]=false;gAuthInfo[Common.AUTHINFO_IS_FIRST_CERT_AUTH]=false;var e=null;gAuthInfo[Common.AUTHINFO_CURR_AUTH]=resObj["curAuth"];switch(resObj["curAuth"]){case Common.CERT_AUTH:e=Common.EVENT_MAIN_CERT_AUTH_ERROR;break;case Common.PASSWORD_AUTH:e=Common.EVENT_MAIN_PSW_AUTH_ERROR;break;case Common.SMS_AUTH:case Common.DO_SMS_AUTH:e=Common.EVENT_MAIN_SMS_AUTH_ERROR;break;
case Common.SMS_BOUND_PHONE_AUTH:e=Common.EVENT_SMS_BOUND_PHONE_AUTH_ERROR;break;case Common.HARDWARE_ID_AUTH:e=Common.EVENT_MAIN_HARDWARE_ID_AUTH_ERROR;break;case Common.CHALLENGE_AUTH:e=Common.EVENT_MAIN_CHALLENGE_AUTH_ERROR;break;case Common.TOKEN_AUTH:e=Common.EVENT_MAIN_TOKEN_AUTH_ERROR;break}var errPat=/Location(\s*)=(\s*)(.*pwp_errorcode(\s*)=(\s*)(\d+).*)/;var errMatches=errPat.exec(resObj["note"]);Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"errMatches: "+errMatches);if(errMatches!=null){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,
"match successfully");var urlStr=errMatches[3];Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"urlStr: "+urlStr);gAuthInfo.modifyPsw.errCode=parseInt(errMatches[6]);Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"urlStr: "+urlStr+" errCode: "+gAuthInfo.modifyPsw.errCode);(new SubmitNewPsw).getPage(urlStr+"\x26ischangepwd\x3d1")}else if(resObj["curAuth"]===Common.HARDWARE_ID_AUTH)if(resObj["note"]==null||resObj["note"]===""||resObj["note"].indexOf(Common.HARD_ID_AUTH_ERRORINFO["failVerifyHID"][gLanguage])>=0){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,
"gAuthInfo: "+JSON.stringify(gAuthInfo));if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,Common.AUTH_WINDOWS[gAuthInfo["curAuth"]],Common.HARDID_SUBMIT_WINDOW)}else{ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Prompt"),message:resObj["note"]});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}else{ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:resObj["note"]});if(resObj["curAuth"]===
Common.PASSWORD_AUTH)Cookies.get({url:gAuthInfo["authUrl"],name:"ENABLE_RANDCODE"},function(error,cookies){gPreAuth[Common.PRE_AUTH_RNDIMG]=false;if(error){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"gCookies: error"+error);gPreAuth[Common.PRE_AUTH_RNDIMG]=false}else if(cookies.length>0)if(cookies[0]!=null&&cookies[0]["value"]==="1")gPreAuth[Common.PRE_AUTH_RNDIMG]=true;if(gPreAuth[Common.PRE_AUTH_RNDIMG])(new RandCodeAuth).doRequestAuth(gAuthInfo[Common.AUTHINFO_AUTH_URL]);gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,
Common.AUTH_WINDOW,Common.AUTH_WINDOW)});else{var arg=null;if(resObj["curAuth"]===Common.DO_SMS_AUTH){if(resObj["note"].indexOf(Common.SMS_AUTH_ERRORINFO["codeExpired"][gLanguage])!==-1)(new SmsAuth).getSmsPage(gAuthInfo[Common.AUTHINFO_AUTH_URL],Common.SMS_AUTH_NOT_NEED_LOAD_URL)}else if(resObj["curAuth"]===Common.SMS_BOUND_PHONE_AUTH)if(resObj["note"].indexOf(Common.SMS_AUTH_ERRORINFO["codeExpired"][gLanguage])!==-1)arg=gAuthInfo[Common.AUTHINFO_DISABLE_TIME];Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,
"Sending ipc_main_event: "+e+"...");if(gECWindow!=null)gECWindow.getWebContents().send(e,arg)}}};
AuthController.prototype.doAuthOK=function(resObj){if(typeof resObj!=="object"||resObj==null)return;if(resObj["curAuth"]===Common.PASSWORD_AUTH||resObj["curAuth"]===Common.CERT_AUTH){var currStartAuth=gControllers["conf"].get("curr_start_auth");if(currStartAuth==null)gControllers["conf"].set("curr_start_auth",resObj["curAuth"])}gControllers.conf.set("last_start_auth",gControllers.conf.get("curr_start_auth"));gControllers.conf.set("curr_start_auth",null);gControllers["conf"].save();const singleEC=
App.makeSingleInstance((argv,workingDir)=>{});if(singleEC){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:ECI18n("Failed to login in with this user account, for a user is online!")});if(gECWindow!=null)gECWindow.hide();(new Logout(null)).doRequestAuth(gAuthInfo["authUrl"],null)}else{gCurrState=Common.STATE_SET_TWFID;var urlObj=Url.parse(gAuthInfo["authUrl"]);gControllers["httpsCom"].requestECAgent(Common.HTTPS_DO_CONFIGURE,{arg1:"SET SERVADDR "+urlObj.hostname+" "+(urlObj.port?
urlObj.port:"443"),token:gAuthInfo["currToken"],callback:Common.CONFIGURE_SERV_ADDR_CALLBACK})}};
AuthController.prototype.showPswRemind=function(){var other=null;if("Conf"in gConf&&"Other"in gConf.Conf)other=gConf.Conf.Other;if(other==null)return;if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_PROGRESS_CHANGE,Common.PROGRESS_STOP);if(other["isPubUser"]!=="1"&&other["pwp_remind_msg"]!=null&&other["pwp_remind_msg"]!="")ECUtil.showMessageBox(null,{type:"info",title:ECI18n("Prompt"),message:other["pwp_remind_msg"]});if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_PROGRESS_CHANGE,
Common.PROGRESS_START)};
AuthController.prototype.startService=function(){gCurrState=Common.STATE_START_SERVICE;if("Conf"in gConf&&"SysTray"in gConf.Conf&&"enable"in gConf.Conf.SysTray&&gConf.Conf.SysTray.enable==="1")gStartTray=true;if(!gStartTray&&gStartFromECAgent)ECUtil.exit(0);if(gECTray==null){var ECTray=require("../windows/js/ec_tray");var trayType=gControllers.conf.getUrl(null,"tray_type");if(gOS===Common.SYS_LINUX&&trayType==="float")ECTray=require("../windows/js/float_window");gECTray=new ECTray(Common.TRAY_HAVE_ICO)}gControllers.httpsCom.requestECAgent(Common.HTTPS_START_SERVICE,{arg1:"EasyConnect",
callback:Common.START_SERVICE_CALLBACK})};AuthController.prototype.doStartServiceOk=function(){global.gCurrTimes=0;global.gIsAllServiceOk=false;global.gIsL3VPNOk=false;if(!gAuthInfo.needStartL3VPN)gIsL3VPNOk=true;global.gQuerryStateTimer=setInterval(this.timerCallBack,Common.QUERRY_STATE_TIMER_INTERVAL)};
AuthController.prototype.timerCallBack=function(){gCurrTimes++;var needOpenRcWindow=true;if(gConf!=null&&"Conf"in gConf&&"Other"in gConf.Conf&&"enable_cs_rc_windows"in gConf.Conf.Other&&gConf.Conf.Other.enable_cs_rc_windows==0)needOpenRcWindow=false;if(gIsAllServiceOk&&gIsL3VPNOk){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Allservices and L3VPN are OK!");gFloatWindowStartServiceOk=true;if(gQuerryStateTimer!=null){clearInterval(gQuerryStateTimer);gQuerryStateTimer=null;gCurrState=Common.STATE_TRAY;
if(gECWindow!=null){gECWindow.close();gECWindow=null}if(gECTray==null){var ECTray=require("../windows/js/ec_tray");var trayType=gControllers.conf.getUrl(null,"tray_type");if(gOS===Common.SYS_LINUX&&trayType==="float")ECTray=require("../windows/js/float_window");gECTray=new ECTray(Common.TRAY_HAVE_ICO)}gECTray.init();if(gECTray!=null)gECTray.setTrayStatus(Common.TRAY_NORMAL);if(!gStartFromECAgent&&!this.mOpenResources&&needOpenRcWindow){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Open Resources!");this.mOpenResources=
true;ECUtil.openExternalUrl(null,gAuthInfo["authUrl"]+"/por/shortcut.csp?type\x3dcs\x26destUrl\x3dservice.csp\x26autoOpen\x3d1\x26language\x3d"+gLanguage+"\x26is_reminded\x3d1\x26ea_port\x3d"+gECAgent["port"]+"\x26id\x3d"+gAuthInfo["currToken"])}if(gControllers.sessionQuerryTimer==null){gControllers["sessionQuerryTimer"]=new SessionQuerryTimer;gControllers["sessionQuerryTimer"].init()}}return}if(gCurrTimes===3&&gQuerryStateTimer!=null){if(gECWindow!=null)gECWindow.hide();if(gECTray==null){var ECTray=
require("../windows/js/ec_tray");var trayType=gControllers.conf.getUrl(null,"tray_type");if(gOS===Common.SYS_LINUX&&trayType==="float")ECTray=require("../windows/js/float_window");gECTray=new ECTray(Common.TRAY_HAVE_ICO)}gECTray.init();if(!gStartFromECAgent&&!this.mOpenResources&&needOpenRcWindow){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"Open Resources!");this.mOpenResources=true;ECUtil.openExternalUrl(null,gAuthInfo["authUrl"]+"/por/shortcut.csp?type\x3dcs\x26destUrl\x3dservice.csp\x26autoOpen\x3d1\x26language\x3d"+
gLanguage+"\x26is_reminded\x3d1\x26ea_port\x3d"+gECAgent["port"]+"\x26id\x3d"+gAuthInfo["currToken"])}}if(!gIsAllServiceOk)gControllers.httpsCom.requestECAgent(Common.HTTPS_DO_QUERY,{arg1:"QUERY QSTATE ALLSERVICES",callback:Common.QUERRY_QSTATE_ALLSERVICES_CALLBACK});if(!gIsL3VPNOk)gControllers.httpsCom.requestECAgent(Common.HTTPS_DO_QUERY,{arg1:"QUERY QSTATE L3VPN",callback:Common.QUERRY_QSTATE_L3VPN_CALLBACK})};
AuthController.prototype.doStartServiceLogout=function(){if(gQuerryStateTimer!=null){clearInterval(gQuerryStateTimer);gQuerryStateTimer=null;ECUtil.exit(0)}};
AuthController.prototype.doStartServiceError=function(){if(gQuerryStateTimer!=null)if(gStartFromECAgent)ECUtil.exit(1);else{if(gECTray!=null){gECTray.destroy();gECTray=null}gControllers.httpsCom.requestECAgent("Logout",{callback:"this.logoutCallback"});ECEvent.once(Common.EVENT_START_SERVICE_ERROR_LOGOUT_OK,function(){ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:ECI18n("Service failed to start. Please try logining in again.")})});(new Logout(Common.LOGOUT_START_SERVICE_ERROR)).doRequestAuth(gAuthInfo["authUrl"],
null);if(gECWindow!=null){gECWindow.show();gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOW)}}if(gQuerryStateTimer!=null){clearInterval(gQuerryStateTimer);gQuerryStateTimer=null}};
AuthController.prototype.setTWFIDToECAgent=function(){var queryObj={};if(gECAgent["encryptKey"]==null||gECAgent["encryptKey"]==""){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The encryptKey of ECAgent is null!");return}if(gAuthInfo["currToken"]==null||gAuthInfo["currToken"]==""){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The currToken is null!");return}var urlObj=Url.parse(gAuthInfo["authUrl"]);gControllers["httpsCom"].requestECAgent(Common.HTTPS_INIT_ECAGENT,{arg1:urlObj["hostname"]+" "+(urlObj["port"]!=
null?urlObj.port:"443"),token:gAuthInfo["currToken"]});gControllers.httpsCom.requestECAgent(Common.HTTPS_GET_ENCRYPT_KEY,{callback:Common.GET_ENCRYPT_KEY_FOR_CERT_AND_TWFID});ECEvent.removeAllListeners(Common.EVENT_GET_ENCRYPT_KEY_FOR_CERT_AND_TWFID_RES);ECEvent.once(Common.EVENT_GET_ENCRYPT_KEY_FOR_CERT_AND_TWFID_RES,(res)=>{if(res!=Common.GET_ENCRYPT_KEY_OK){ECEvent.emit(Common.EVENT_AUTH_ERROR,Common.AUTH_GET_ENCRYPT_KEY_ERROR);return}var tmpToken=null,encryptKey=gECAgent["encryptKey"],encryptExp=
parseInt("65537").toString(16);var rsaKey=new RSAKey;rsaKey.setPublic(encryptKey,encryptExp);tmpToken=rsaKey.encrypt(gAuthInfo["currToken"]);queryObj["arg1"]="SET TWFID "+tmpToken;queryObj["callback"]=Common.CONFIGURE_TWFID;gControllers["httpsCom"].requestECAgent(Common.HTTPS_DO_CONFIGURE,queryObj)})};
AuthController.prototype.doNextAuth=function(resObj){if(typeof resObj!=="object"||resObj==null)return;gAuthInfo["curAuth"]=resObj["curAuth"];gAuthInfo["nextAuth"]=resObj["nextAuth"];Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"The next auth is "+resObj["nextAuth"]);var authUrl=this.getWindowUrl(resObj["nextAuth"]);if(authUrl==null){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"Donot support the auth type!");ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:ECI18n("Unknown authentication type")});
return}if(resObj["curAuth"]===Common.PASSWORD_AUTH||resObj["curAuth"]===Common.CERT_AUTH){var currStartAuth=gControllers["conf"].get("curr_start_auth");if(currStartAuth==null)gControllers["conf"].set("curr_start_auth",resObj["curAuth"])}if(resObj["nextAuth"]===Common.CHALLENGE_AUTH)gAuthInfo["challengeMsg"]=resObj["challengeMsg"];if(resObj["nextAuth"]===Common.SMS_AUTH){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"next auth is sms_auth");this.doSmsAuth(resObj)}else if(resObj["nextAuth"]===Common.HARDWARE_ID_AUTH){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,
"gHardID is null");gAuthInfo["hardID"]={};setTimeout(()=>{gControllers["httpsCom"].requestECAgent(Common.HTTPS_DO_QUERY,{arg1:"QUERY GETHARDID",token:gAuthInfo["currToken"],callback:Common.QUERRY_HARDID_CALLBACK})},1E3)}else if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,null,Common.AUTH_WINDOWS[resObj["nextAuth"]])};
AuthController.prototype.doSmsAuth=function(resObj){if(resObj==null||resObj["phone"]==null||resObj["phone"]===""){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"sms_auth_submit_phone_window: "+Common.SMS_AUTH_SUBMIT_PHONE_WINDOW_URL);if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,Common.AUTH_WINDOWS[resObj["curAuth"]],Common.AUTH_WINDOWS[resObj["nextAuth"]]["phone"])}else{gAuthInfo["phone"]=resObj["phone"];gAuthInfo["smsIsStillValid"]=resObj["smsIsStillValid"];gAuthInfo["note"]=
resObj["note"];if(gAuthInfo["phone"].indexOf(";")==-1)setTimeout(()=>{(new SmsAuth).getSmsPage(gAuthInfo["authUrl"],Common.SMS_AUTH_NEED_LOAD_URL)},10);else if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_CHANGE_WINDOW,Common.AUTH_WINDOWS[gAuthInfo["curAuth"]],Common.SMS_WINDOW)}};
AuthController.prototype.doSubmitPhoneRes=function(resStr){if(resStr==null||""==resStr){Log.warning(Common.LOG_AUTH_CONTROLLER_TAG,"The submit_phone_result is null!");return}var resObj=JSON.parse(resStr);var authInfo={};switch(resObj["result"]){case Common.SMS_SUBMIT_PHONE_OK:authInfo["disableTime"]=resObj["smsIsStillValid"];gAuthInfo[Common.AUTHINFO_DISABLE_TIME]=resObj["smsIsStillValid"];Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"sms_auth: "+Common.SMS_SUBMIT_PHONE_OK);if(gECWindow!=null)if(resObj["type"]==
Common.SMS_SUBMIT_PHONE)gECWindow.getWebContents().send(Common.EVENT_MAIN_SUBMIT_PHONE_OK,JSON.stringify(authInfo));else gECWindow.getWebContents().send(Common.EVENT_MAIN_SMS_DISABLE_TIME,resObj["smsIsStillValid"]);break;default:authInfo["note"]=resObj["note"];authInfo["disableTime"]=0;Log.info(Common.LOG_AUTH_CONTROLLER_TAG,resObj["note"]);ECUtil.showMessageBox(null,{type:"warning",title:ECI18n("Alert"),message:resObj["note"]});if(gECWindow!=null)if(resObj["type"]==Common.SMS_SUBMIT_PHONE)gECWindow.getWebContents().send(Common.EVENT_MAIN_SUBMIT_PHONE_ERROR,
JSON.stringify(authInfo));else gECWindow.getWebContents().send(Common.EVENT_MAIN_SMS_DISABLE_TIME,0);break}};AuthController.prototype.doRegetSmsCodeRes=function(resStr){Log.debug(Common.LOG_AUTH_CONTROLLER_TAG,"type of reget_sms_code_res: "+typeof resStr);if(gECWindow!=null)gECWindow.getWebContents().send(Common.EVENT_MAIN_REGET_SMS_CODE_RESULT,resStr)};
AuthController.prototype.getWindowUrl=function(type){switch(type){case Common.CERT_AUTH:return Common.AUTH_WINDOW_URL;case Common.PASSWORD_AUTH:return Common.AUTH_WINDOW_URL;case Common.SMS_AUTH:return Common.SMS_AUTH_WINDOW_URL;break;case Common.HARDWARE_ID_AUTH:return Common.HARD_ID_AUTH_WINDOW_URL;case Common.USBKEY_AUTH:return null;case Common.CHALLENGE_AUTH:return Common.CHALLENGE_AUTH_WINDOW_URL;case Common.TOKEN_AUTH:return Common.TOKEN_AUTH_WINDOW_URL;default:return null}};
module.exports=AuthController;
